/*********************************************************************
	Rhapsody in C	: 8.2.1 
	Login		: LocalAdministrator
	Component	: Cortex_CMP 
	Configuration 	: Debug
	Model Element	: SPI
//!	Generated Date	: Sat, 19, Aug 2017  
	File Path	: Cortex_CMP\Debug\SPI.c
*********************************************************************/

/*## auto_generated */
#include "SPI.h"
/*## package a_SPI_Lib */

/*## class TopLevel::SPI */

/*#[ ignore */

/* Beginning start section to allocate CODE and DATA into Memory Sections */

#define WST_APPL_RUNNING_START_SEC_DATA_NOINIT_UNSPECIFIED_FAST
#include "WSTMemoryTypes.h"

#define WST_APPL_RUNNING_START_SEC_CODE_FAST
#include "WSTMemoryTypes.h"

/* Ending start section to allocate CODE and DATA into Memory Sections */


/*#]*/


/*## operation check_ctrl() */
static void check_ctrl( void );

/* Read data from the LCD controller  */
/*    Parameter: */
/*    Return:               read data  */
/*## operation rd_dat() */
static uint16_t rd_dat( void );

/* Read from the LCD register */
/*    Parameter:    reg:    register to be read  */
/*    Return:               value read from the register */
/*  */
/*## operation rd_reg(uint8_t) */
static uint16_t rd_reg(const uint8_t aReg);

/* Transfer 1 byte over the serial communication */
/*    Parameter:    byte:   byte to be sent */
/*    Return:               byte read while sending */
/*## operation spi_tran(uint8_t) */
static uint8_t spi_tran(const uint8_t aByte);

/*## operation spi_tran_man(uint8_t,datamode_t) */
static uint16_t spi_tran_man(const uint8_t aByte, const datamode_t aMode);

/*## operation Init() */
void SPI_Init( void )
{
    SPI.hiMax = 0;
    /*#[ operation Init() */
    /*#]*/
}

void SPI_Cleanup( void )
{
}

/*## operation delay(uint32_t) */
void SPI_delay(uint32_t aCnt)
{
    /*#[ operation delay(uint32_t) */
    aCnt <<= DELAY_2N;
    while ( aCnt-- );
    
    /*#]*/
}

/*## operation initialize() */
void SPI_initialize( void )
{
    /*#[ operation initialize() */
    /* Enable clock for SSP1, clock = CCLK / 2                                  */
    LPC_SC->PCONP       |= 0x00000400;
    LPC_SC->PCLKSEL0    |= 0x00200000;
    
    /* Configure the LCD Control pins                                           */
    LPC_PINCON->PINSEL9 &= 0xF0FFFFFF;
    LPC_GPIO4->FIODIR   |= 0x30000000;
    LPC_GPIO4->FIOSET    = 0x20000000;
    
    /* SSEL1 is GPIO output set to high                                         */
    LPC_GPIO0->FIODIR   |= 0x00000040;
    LPC_GPIO0->FIOSET    = 0x00000040;
    LPC_PINCON->PINSEL0 &= 0xFFF03FFF;
    LPC_PINCON->PINSEL0 |= 0x000A8000;
    
    /* Enable SPI in Master Mode, CPOL=1, CPHA=1                                */
    /* Max. 12.5 MBit used for Data Transfer @ 100MHz                           */
    LPC_SSP1->CR0        = 0x01C7;
    LPC_SSP1->CPSR       = 0x02;
    LPC_SSP1->CR1        = 0x02;
    
    check_ctrl();
    
    /*#]*/
}

/*## operation isHiMax() */
RiCBoolean SPI_isHiMax( void )
{
    /*#[ operation isHiMax() */
    return SPI.hiMax;
    
    /*#]*/
}

/*## operation rd_id_man() */
int16_t SPI_rd_id_man( void )
{
    /*#[ operation rd_id_man() */
    unsigned short val;
    
    /* Set MOSI, MISO and SCK as GPIO pins, with pull-down/pull-up disabled     */
    LPC_PINCON->PINSEL0  &= ~((3 << 18) | (3 << 16) | (3 << 14));
    LPC_PINCON->PINMODE0 |= 0x000AA000;
    LPC_GPIO0->FIODIR    |= PIN_CLK;      /* SCK pin is GPIO output             */
    
    LCD_CS (1);                           /* Set chip select high               */
    LCD_CLK(1);                           /* Set clock high                     */
    
    LCD_CS(0);
    spi_tran_man ( SPI_START | SPI_WR | SPI_INDEX, DM_OUT);
    spi_tran_man ( 0x00, DM_OUT);
    LCD_CS(1);
    
    LCD_CS(0);
    spi_tran_man ( SPI_START | SPI_RD | SPI_DATA, DM_OUT);
    val = spi_tran_man( 0, DM_IN);
    LCD_CS(1);
    
    /* Connect MOSI, MISO, and SCK to SSP peripheral                            */
    LPC_GPIO0->FIODIR    &= ~PIN_CLK;
    LPC_PINCON->PINSEL0  |= (2 << 18) | (2 << 16) | (2 << 14);
    LPC_PINCON->PINMODE0 &= ~0x000FF000;
    
    return (val);
    
    /*#]*/
}

/*## operation wr_cmd(uint8_t) */
void SPI_wr_cmd(const uint8_t aCmd)
{
    /*#[ operation wr_cmd(uint8_t) */
    LCD_CS(0);
    
    spi_tran( SPI_START | SPI_WR | SPI_INDEX );   /* Write : RS = 0, RW = 0       */
    spi_tran( 0 );
    spi_tran( aCmd );
    
    LCD_CS(1);
    
    /*#]*/
}

/*## operation wr_dat(uint16_t) */
void SPI_wr_dat(const uint16_t aDat)
{
    /*#[ operation wr_dat(uint16_t) */
    LCD_CS(0);
    
    spi_tran( SPI_START | SPI_WR | SPI_DATA );     /* Write : RS = 1, RW = 0       */
    spi_tran( (aDat >>   8) );                     /* Write D8..D15                */
    spi_tran( (aDat & 0xFF) );                     /* Write D0..D7                 */
    
    LCD_CS(1);
    
    /*#]*/
}

/*## operation wr_dat_only(uint16_t) */
void SPI_wr_dat_only(const uint16_t aDat)
{
    /*#[ operation wr_dat_only(uint16_t) */
    /* Write D8..D15                */
    spi_tran( ( aDat >>   8 ) );
    
    /* Write D0..D7                 */
    spi_tran( ( aDat & 0xFF ) );
    
    /*#]*/
}

/*## operation wr_dat_start() */
void SPI_wr_dat_start( void )
{
    /*#[ operation wr_dat_start() */
    LCD_CS(0);
    
    /* Write : RS = 1, RW = 0       */
    spi_tran( SPI_START | SPI_WR | SPI_DATA );
    
    /*#]*/
}

/*## operation wr_dat_stop() */
void SPI_wr_dat_stop( void )
{
    /*#[ operation wr_dat_stop() */
    LCD_CS(1);
    
    /*#]*/
}

/*## operation wr_reg(uint8_t,uint16_t) */
void SPI_wr_reg(const uint8_t aReg, const uint16_t aVal)
{
    /*#[ operation wr_reg(uint8_t,uint16_t) */
    SPI_wr_cmd( aReg );
    SPI_wr_dat( aVal );
    
    /*#]*/
}

/*## operation check_ctrl() */
static void check_ctrl( void )
{
    /*#[ operation check_ctrl() */
    /* Check the Type of LCD Controller */
    SPI.driverCode = SPI_rd_id_man ();
    if (SPI.driverCode == 0) 
    {
    	SPI.driverCode = rd_reg( 0x00);
    }
    
    if ( SPI.driverCode == 0x47)
    {
    	SPI.hiMax = 1;                          /* Set Himax LCD controller flag      */
    }         
    
    /*#]*/
}

/*## operation rd_dat() */
static uint16_t rd_dat( void )
{
    /*#[ operation rd_dat() */
    unsigned short val = 0;
    
    LCD_CS(0);
    
    spi_tran(SPI_START | SPI_RD | SPI_DATA);    /* Read: RS = 1, RW = 1         */
    spi_tran(0);                                /* Dummy read 1                 */  
    val   = spi_tran(0);                        /* Read D8..D15                 */
    val <<= 8;
    val  |= spi_tran(0);                        /* Read D0..D7                  */
    
    LCD_CS(1);
    
    return (val);
    
    /*#]*/
}

/*## operation rd_reg(uint8_t) */
static uint16_t rd_reg(const uint8_t aReg)
{
    /*#[ operation rd_reg(uint8_t) */
    SPI_wr_cmd( aReg );
    return( rd_dat() );
    /*#]*/
}

/*## operation spi_tran(uint8_t) */
static uint8_t spi_tran(const uint8_t aByte)
{
    /*#[ operation spi_tran(uint8_t) */
    LPC_SSP1->DR = aByte;
    
    while (!(LPC_SSP1->SR & RNE));        /* Wait for send to finish            */
    
    return (LPC_SSP1->DR);
    
    /*#]*/
}

/*## operation spi_tran_man(uint8_t,datamode_t) */
static uint16_t spi_tran_man(const uint8_t aByte, const datamode_t aMode)
{
    /*#[ operation spi_tran_man(uint8_t,datamode_t) */
    unsigned char val = 0;
    int i;
    
    if (aMode == DM_OUT) 
    {
    	DAT_MODE (DM_OUT);
    }
    else
    {
    	DAT_MODE (DM_IN);
    }
    
    for (i = 7; i >= 0; i--)
    {
    	LCD_CLK(0);
    	SPI_delay(1);
    
    	if (aMode == DM_OUT) 
    	{
    		LCD_DAT((aByte & (1 << i)) != 0);
    	}
    	else 
    	{
    		val |= (BUS_VAL() << i);
    	}
    
    	LCD_CLK(1);
    	SPI_delay(1);
    }
    return (val);
    
    /*#]*/
}

int16_t SPI_getDriverCode( void )
{
    return SPI.driverCode;
}


/*#[ ignore */

/* Beginning stop section to allocate CODE and DATA into Memory Sections */

#define WST_APPL_RUNNING_STOP_SEC_DATA_NOINIT_UNSPECIFIED_FAST
#include "WSTMemoryTypes.h"

#define WST_APPL_RUNNING_STOP_SEC_CODE_FAST
#include "WSTMemoryTypes.h"

/* Ending stop section to allocate CODE and DATA into Memory Sections */


/*#]*/

/*********************************************************************
	File Path	: Cortex_CMP\Debug\SPI.c
*********************************************************************/
